/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;
using System.Collections;

using System.Text.RegularExpressions;

//using Phidgets;

namespace MobiServer.Devices.Phidget.Control
{
    public partial class Base : UserControl
    {
        public const string SETTING_DIR = "setting";

        protected Phidgets.Phidget phidget = null;

        //PhidgetServer̒`
        protected MobiServer.TCPServer server = null;
        //Serveȑׂ͂ĂPhidgetsɓnKvD

        public event MobiServer.ReadHandler serverReadHandler;
        public event MobiServer.WriteHandler serverWriteHandler;
        public event MobiServer.MessageHandler messageHandler;


        public Base()
        {
            InitializeComponent();
        }

        //public PhidgetControl(string name, string description, 
        //                        int serialNumber, int version, string label)

        //, PhidgetServer.PhidgetServer server

        public Base(Phidgets.Phidget phidget)
        {
            InitializeComponent();
            this.phidget = phidget;
            //this.server = server;
            //this.groupBox1.Text = this.DeviceType + ":" + this.SerialNumber.ToString();
            SetTitle();
        }

        public void SetTitle()
        {
            this.groupBox1.Text = this.DeviceType + ":" + this.SerialNumber.ToString();
        }

        /*public Base(Phidgets.Phidget phidget, PhidgetServer server, messageHandler messageHandler)
        {
            InitializeComponent();
            this.phidget = phidget;
            this.server = server;
            this.messageHandler += messageHandler;
            this.groupBox1.Text = this.DeviceType + ":" + this.SerialNumber.ToString();
        }*/

        //public Point Position
        //{
        //    get { return this.position; }
        //    set { this.position = value; }
        //}

        public string DeviceName
        {
            get { return this.phidget.Name; }
        }

        public string DeviceType
        {
            get { return this.phidget.Type; }
        }

        public int SerialNumber
        {
            get { return this.phidget.SerialNumber; }
        }
        public int Version
        {
            get { return this.phidget.Version; }
        }
        public string Label
        {
            get { return this.phidget.Label; }
        }

        public MobiServer.TCPServer Server
        {
            set { this.server = value; }
            get { return this.server; }
        }


        public virtual bool Start()
        {
            return false;
        }

        public void Stop(){
            phidget.close();
            phidget = null;
        }


        #region bZ[WCxĝ߂̃fQ[g/\bh

        //MobiServerEventArgs邽߂Delgate
        //public delegate void messageHandler(object sender, MobiServerEventArgs e);
        //public delegate void messageHandler(object sender, MobiServerEventArgs e);
        

        protected void PrintType()
        {
            PrintDebug("{0}:{1}\n", phidget.Type, phidget.SerialNumber);
        }

        //protected void PrintType(string m, params object[] args)
        //{
        //    
        //    PrintMessage(String.Format(m, args) + ": {0}:{1}\n", phidget.Type, phidget.SerialNumber);
        //}

        //G[CxgND
        public void PrintError(string m, params object[] args)
        {
            MobiServer.MessageEventArgs e = new MobiServer.MessageEventArgs(String.Format(m, args), MobiServer.States.Error);

            if (messageHandler != null)
                messageHandler(this, e);
        }

        //bZ[WCxgND
        public void PrintMessage(string m, params object[] args)
        {
            MobiServer.MessageEventArgs e = new MobiServer.MessageEventArgs(String.Format(m, args));

            if (messageHandler != null)
                messageHandler(this, e);

            return;
        }

        public void PrintDebug(string m, params object[] args)
        {
            MobiServer.MessageEventArgs e = new MobiServer.MessageEventArgs(String.Format(m, args), MobiServer.States.Debug);

            if (messageHandler != null)
                messageHandler(this, e);
        }

        #endregion

        protected bool CheckDeviceAttached()
        {
            if (!phidget.Attached)
            {
                PrintError("Not found: {0}:{1}", DeviceType, Version);
                PrintType();
                return false;
            }
            else
            {
                return true;
            }
        }
        protected bool CheckDeviceType(Data.Base Data)
        {
            bool result = false;
            if (Data.Type == this.DeviceType)
            {
                if (Data.SerialNumber == 0 ||
                    Data.SerialNumber == this.SerialNumber)
                {
                    result = true;
                }
            }
            return result;

        }

        protected bool CheckDeviceType(string [] args)
        {
            bool result = false;

            try
            {
                Data.Base p = Data.Base.Parse(args);

                Match m = Regex.Match(p.Type, this.DeviceType, RegexOptions.IgnoreCase);
                             
                //if (p.Type == this.DeviceType)
                if(m.Success)
                {
                    if (p.SerialNumber == 0 ||
                        p.SerialNumber == this.SerialNumber)
                    {
                        result = true;
                    }
                }

                /*
                string[] args = data.Split(Data.Base.DELIMITER_ARGS.ToCharArray());
                
                if (args != null && args.Length > Data.Base.ARGS_LENGTH_MIN && 
                    args[0] == Data.Base.HEADER)
                {
                    string type = (args[1]);
                    int serialNumber = int.Parse(args[2]);
                    if (type == this.DeviceType)
                    {
                        if (serialNumber == 0 ||
                            serialNumber == this.SerialNumber)
                        {
                            result = true;
                        }
                    }
                }
                 * */

            }
            catch
            {

            }
            return result;
        }



        protected string GetSettingFile()
        {
            string dir = Application.StartupPath + @"\" + SETTING_DIR;

            if (!Directory.Exists(dir))
            {
                Directory.CreateDirectory(dir);
            }

            string filename = String.Format(@"{0}\{1}_{2}.txt", dir, DeviceType, SerialNumber);

            return filename;
        }


        public virtual void SaveSetting()
        {

        }

        public virtual void LoadSetting()
        {

        }

        public virtual void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {

        }

        //public void CreateServerWriteEvent(Data.Base Data)
        //public void CreateServerWriteEvent(string data)    
        public void CreateServerWriteEvent(string[] args)
        {
            MobiServer.ServerEventArgs e = new MobiServer.ServerEventArgs(args);//Data
            serverWriteHandler.Invoke(this, e);
        }

        /*//C#ŒڃCxgꍇCȉҏWDi݂͖gpj
        protected void DealEvent(Data phidget)//PhidgetServer. phidget
        {
            

            switch (phidget.Type)
            {
                case PhidgetControlData.Types.InterfaceKit:
                    Phi InterfaceKit interfaceKit = 
                                            ((PhidgetControlData.Types.InterfaceKit)phidget);
                    break;
                case PhidgetControlData.Types.RFID:
                    Command.RFID rfid = ((Command.RFID)phidget);
                    break;
                case Command.Types.Accel:
                    Command.Accel accel = ((Command.Accel)phidget);
                    break;
                case Command.Types.Weight:
                    Command.Weight weight = ((Command.Weight)phidget);
                    break;
            }
             
           
            return;
        }
         */

        //͕KvȂ̂ł͂ȂCƎvD
        //ݐڑĂ̂N邽тɌĂяo΂悢CD
        //āCf[^͕̂ۑĂCꏊȂǂׂ͂Ė̂悢CD
        /*public string[] GetSettingFiles()
        {
            string dir = Application.StartupPath + @"\" + SETTING_DIR;
            Hashtable phidgetHolder = new Hashtable();


            if (Directory.Exists(dir))
            {
                Directory.CreateDirectory(dir);
            }

            string[] files = Directory.GetFiles(dir);

            return files;

        }*/

  
        /*
        protected bool SaveSettingPosition(StreamWriter writer)
        {
            bool result = false;

            //string filenamePosition = Path.GetFileNameWithoutExtension(filename) + "_position." +  Path.GetExtension(filename);
            //StreamWriter fileStream = null;
            
            try
			{
                if (writer != null && writer.BaseStream.CanRead)
                {

                    //fileStream = new StreamWriter(filenamePosition, false);
                    writer.WriteLine(String.Format("{0}\t{1}\t{2}", position.X, position.Y, Enabled));
                    result = true;
                    //fileStream.WriteLine(this.Enabled.ToString());
                }
			}
			catch(IOException ex)
			{
				PrintError(ex.Message);
			}
            catch (Exception ex)
            {
                PrintError("Saving " + file);

            }

            return result;

        }

        protected bool LoadSettingPosition(StreamReader reader)
        {
            //string filenamePosition = Path.GetFileNameWithoutExtension(filename) + "_position." +  Path.GetExtension(filename);
            //StreamReader fileStream = null;

            //t@CȂΏI
			//if(!File.Exists(filenamePosition))
			//    return;

            bool result = false;
            
            try
			{
                if (reader != null && reader.BaseStream.CanRead)
                {
                    //fileStream = new StreamReader(filenamePosition);

                    string line = reader.ReadLine();

                    //if (line == null)
                    //    return;

                    string[] args = line.Split(@"\t");

                    //if (args.Length < 3)
                    //    return;

                    this.Position = new Point(int.Parse(args[0]), int.Parse(args[1]));
                    this.Enabled = bool.Parse(args[2]);

                    result = true;
                }
			}
			catch(IOException ex)
			{
				PrintError(ex.Message);
			}
            catch (Exception ex)
            {
                PrintError("Loading " + file);

            }

            return result;
			//finally
			//{
			//	if(fileStream != null)
			//	{
			//		fileStream.Close();
			//		fileStream = null;
			//	}
			//}

        }
         * */

       
        //protected virtual void  SaveSetting(){


 //       }




    }
}