/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using System.IO;


namespace MobiServer.Devices.Phidget.Control
{
    //Todo: ő|[g8ȊOւ̑Ή

    public partial class InterfaceKit : Base
    {
     
        Phidgets.InterfaceKit phidgetInterface;
        private CheckBox[] checkBoxInterfaces;
        private CheckBox[] checkBoxInputs;
        private CheckBox[] checkBoxOutputs;
        private NumericUpDown[] numericInterfaces;
        private NumericUpDown[] numericInterfaceTs;

        public InterfaceKit(Phidgets.InterfaceKit phidget) 
            : base((Phidgets.Phidget)phidget)
        {
            InitializeComponent();
            InitComponentArray();
            this.phidgetInterface = phidget;
            LoadSetting();
        }

        void InitComponentArray()
        {
            numericInterfaces = new System.Windows.Forms.NumericUpDown[8];
            numericInterfaces[0] = numericUpDownInterface1;
            numericInterfaces[1] = numericUpDownInterface2;
            numericInterfaces[2] = numericUpDownInterface3;
            numericInterfaces[3] = numericUpDownInterface4;
            numericInterfaces[4] = numericUpDownInterface5;
            numericInterfaces[5] = numericUpDownInterface6;
            numericInterfaces[6] = numericUpDownInterface7;
            numericInterfaces[7] = numericUpDownInterface8;

            numericInterfaceTs = new System.Windows.Forms.NumericUpDown[8];
            numericInterfaceTs[0] = numericUpDownInterfaceT1;
            numericInterfaceTs[1] = numericUpDownInterfaceT2;
            numericInterfaceTs[2] = numericUpDownInterfaceT3;
            numericInterfaceTs[3] = numericUpDownInterfaceT4;
            numericInterfaceTs[4] = numericUpDownInterfaceT5;
            numericInterfaceTs[5] = numericUpDownInterfaceT6;
            numericInterfaceTs[6] = numericUpDownInterfaceT7;
            numericInterfaceTs[7] = numericUpDownInterfaceT8;

            checkBoxInterfaces = new System.Windows.Forms.CheckBox[8];
            checkBoxInterfaces[0] = checkBoxInterface1;
            checkBoxInterfaces[1] = checkBoxInterface2;
            checkBoxInterfaces[2] = checkBoxInterface3;
            checkBoxInterfaces[3] = checkBoxInterface4;
            checkBoxInterfaces[4] = checkBoxInterface5;
            checkBoxInterfaces[5] = checkBoxInterface6;
            checkBoxInterfaces[6] = checkBoxInterface7;
            checkBoxInterfaces[7] = checkBoxInterface8;

            checkBoxInputs = new System.Windows.Forms.CheckBox[8];
            checkBoxInputs[0] = checkBoxInput1;
            checkBoxInputs[1] = checkBoxInput2;
            checkBoxInputs[2] = checkBoxInput3;
            checkBoxInputs[3] = checkBoxInput4;
            checkBoxInputs[4] = checkBoxInput5;
            checkBoxInputs[5] = checkBoxInput6;
            checkBoxInputs[6] = checkBoxInput7;
            checkBoxInputs[7] = checkBoxInput8;

            checkBoxOutputs = new System.Windows.Forms.CheckBox[8];
            checkBoxOutputs[0] = checkBoxOutput1;
            checkBoxOutputs[1] = checkBoxOutput2;
            checkBoxOutputs[2] = checkBoxOutput3;
            checkBoxOutputs[3] = checkBoxOutput4;
            checkBoxOutputs[4] = checkBoxOutput5;
            checkBoxOutputs[5] = checkBoxOutput6;
            checkBoxOutputs[6] = checkBoxOutput7;
            checkBoxOutputs[7] = checkBoxOutput8;

            return;
        }

       
        override public bool Start()
        {
            try
            {
                if (!CheckDeviceAttached())
                    return false;

                PrintType();

                for (int i = 0; i < phidgetInterface.sensors.Count; i++)
                {
                    phidgetInterface.sensors[i].Sensitivity = (int)numericInterfaceTs[i].Value;
                }

                phidgetInterface.SensorChange += new Phidgets.Events.SensorChangeEventHandler(OnInterfaceSensorChange);
                phidgetInterface.InputChange += new Phidgets.Events.InputChangeEventHandler(OnInterfaceInputChange);
            }

            catch (Exception ex)
            {
               PrintError(ex.Message);
               return false;
            }
            return true;
        }

        //public void StopInterface()
        //{
        //    phidgetInterface = null;
        //    foreach (CheckBox box in checkBoxInterfaces)
        //    {
        //        box.Checked = false;
        //    }
        //}

   
        private void OnInterfaceSensorChange(object sender, Phidgets.Events.SensorChangeEventArgs e)
        {
            PrintDebug("InterfaceKit:Sensor:{0}: {1}", e.Index, e.Value);
            checkBoxInterfaces[e.Index].Checked = true;
            numericInterfaces[e.Index].Value = e.Value;
            Application.DoEvents();
            Data.InterfaceKit interfaceData = new Data.InterfaceKit(SerialNumber, 
                                               Data.States.In,
                                               Data.InterfaceKit.SubTypes.Analog,
                                               e.Index,
                                               e.Value);

            CreateServerWriteEvent(interfaceData.ToArray());
        }

        private void OnInterfaceInputChange(object sender,  Phidgets.Events.InputChangeEventArgs e)
        {
            PrintDebug("InterfaceKit:Input:{0}: {1}", e.Index, e.Value);
            checkBoxInputs[e.Index].Checked = e.Value;
            Application.DoEvents();
            int newVal = 0;

            if (e.Value)
                newVal = 1;

            Data.InterfaceKit interfaceData = new Data.InterfaceKit(SerialNumber,
                                                        Data.States.In,
                                                        Data.InterfaceKit.SubTypes.Digital,
                                                        e.Index,
                                                        newVal);

            CreateServerWriteEvent(interfaceData.ToArray());
        }

        private void numericUpDownInterfaceT1_ValueChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.sensors[0].Sensitivity = (int)numericInterfaceTs[0].Value;
            }
        }

        private void numericUpDownInterfaceT2_ValueChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.sensors[1].Sensitivity = (int)numericInterfaceTs[1].Value;
            }
        }

        private void numericUpDownInterfaceT3_ValueChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.sensors[2].Sensitivity =  (int)numericInterfaceTs[2].Value;
            }
        }

        private void numericUpDownInterfaceT4_ValueChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.sensors[3].Sensitivity = (int)numericInterfaceTs[3].Value;
            }
        }

        private void numericUpDownInterfaceT5_ValueChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.sensors[4].Sensitivity = (int)numericInterfaceTs[4].Value;
            }

        }

        private void numericUpDownInterfaceT6_ValueChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.sensors[5].Sensitivity =  (int)numericInterfaceTs[5].Value;
            }
        }

        private void numericUpDownInterfaceT7_ValueChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.sensors[6].Sensitivity = (int)numericInterfaceTs[6].Value;
            }
        }

        private void numericUpDownInterfaceT8_ValueChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.sensors[7].Sensitivity =  (int)numericInterfaceTs[7].Value;
            }
        }

        private void checkBoxOutput1_CheckedChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.outputs[0] = ((CheckBox)sender).Checked;
            }
        }

        private void checkBoxOutput2_CheckedChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.outputs[1] = ((CheckBox)sender).Checked;
            }
        }

        private void checkBoxOutput3_CheckedChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.outputs[2] = ((CheckBox)sender).Checked;
            }
        }

        private void checkBoxOutput4_CheckedChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.outputs[3] = ((CheckBox)sender).Checked;
            }
        }

        private void checkBoxOutput5_CheckedChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.outputs[4] = ((CheckBox)sender).Checked;
            }
        }

        private void checkBoxOutput6_CheckedChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.outputs[5] = ((CheckBox)sender).Checked;
            }
        }

        private void checkBoxOutput7_CheckedChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.outputs[6] = ((CheckBox)sender).Checked;
            }
        }

        private void checkBoxOutput8_CheckedChanged(object sender, System.EventArgs e)
        {
            if (phidgetInterface != null && phidgetInterface.Attached)
            {
                phidgetInterface.outputs[7] = ((CheckBox)sender).Checked;
            }
        }

        override public void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {

            try
            {
                if (CheckDeviceType(e.Args))
                {
                    Data.InterfaceKit ik = Data.InterfaceKit.Parse(e.Args);

                    if (ik != null)
                    {
                        switch (ik.State)
                        {
                            case Data.States.Set:
                                if (ik.Port < phidgetInterface.sensors.Count)
                                    numericInterfaceTs[ik.Port].Value = ik.Data;
                                break;
                            case Data.States.Out:
                                if (ik.Port < phidgetInterface.outputs.Count &&
                                     ik.Port < checkBoxOutputs.Length)
                                {
                                    if (ik.Data == 1)
                                    {
                                        checkBoxOutputs[ik.Port].Checked = true;
                                    }
                                    else if (ik.Data == 0)
                                    {
                                        checkBoxOutputs[ik.Port].Checked = false;
                                    }
                                }
                                break;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }


        #region Save/Load

		override public void SaveSetting()
		{
			StreamWriter fileStream = null;
            string file = GetSettingFile();

            try
            {
                fileStream = new StreamWriter(file, false);

                for (int i = 0; i < numericInterfaceTs.Length; i++)
                {
                    fileStream.Write("{0}\t", numericInterfaceTs[i].Value);
                }
                fileStream.WriteLine();

                for (int i = 0; i < checkBoxOutputs.Length; i++)
                {
                    fileStream.Write("{0}\t", checkBoxOutputs[i].Checked);
                }
                fileStream.WriteLine();
            }
            catch (IOException ex)
            {
                PrintError(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Saving " + Path.GetFileName(file));
            }
			finally
			{
				if(fileStream != null)
				{
					fileStream.Close();
					fileStream = null;
				}
			}
		}

        override public void LoadSetting()
        {
            StreamReader fileStream = null;
            string file = GetSettingFile();

            try
            {
                if (File.Exists(file))
                {

                    fileStream = new StreamReader(file);

                    string line = null;
                    string[] args = null;

                    line = fileStream.ReadLine();
                    args = line.Split("\t".ToCharArray());
                    for (int i = 0; i < numericInterfaceTs.Length; i++)
                    {
                        numericInterfaceTs[i].Value = int.Parse(args[i]);
                    }

                    line = fileStream.ReadLine();
                    args = line.Split("\t".ToCharArray());

                    for (int i = 0; i < checkBoxOutputs.Length; i++)
                    {
                        checkBoxOutputs[i].Checked = bool.Parse(args[i]);
                    }
                }

            }
            catch (IOException ex)
            {
                PrintMessage(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Loading " + Path.GetFileName(file));

            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }
       

        #endregion

      
    }
}

