/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;

namespace MobiServer.Devices.Phidget.Control

{
    public partial class Motor : Base
    {

        TrackBar[] trackBarVelocities= null;
        TrackBar[] trackBarAccels = null;
        CheckBox[] checkBoxInputs = null;
        Phidgets.MotorControl phidgetMotor = null;


        public Motor(Phidgets.Phidget phidget)
            : base(phidget)
        {
            InitializeComponent();
            InitComponentArray();
            phidgetMotor = (Phidgets.MotorControl)phidget;
            LoadSetting();
        }

        void InitComponentArray()
        {
            trackBarVelocities = new System.Windows.Forms.TrackBar[2];
            trackBarVelocities[0] = trackBarVelocity1;
            trackBarVelocities[1] = trackBarVelocity2;

            trackBarAccels = new System.Windows.Forms.TrackBar[2];
            trackBarAccels[0] = trackBarAccel1;
            trackBarAccels[1] = trackBarAccel2;

            checkBoxInputs = new CheckBox[4];
            checkBoxInputs[0] = checkBoxInput0;
            checkBoxInputs[1] = checkBoxInput1;
            checkBoxInputs[2] = checkBoxInput2;
            checkBoxInputs[3] = checkBoxInput3;

            return;
        }

       
        override public bool Start()
        {
        
            try
            {
                if (!CheckDeviceAttached())
                    return false;
                
                PrintType();

                phidgetMotor.motors[0].Velocity = trackBarVelocity1.Value;
                phidgetMotor.motors[0].Acceleration = trackBarAccel1.Value;
                phidgetMotor.motors[1].Velocity = trackBarVelocity2.Value;
                phidgetMotor.motors[1].Acceleration = trackBarAccel2.Value;

                numericUpDownAccel1.Value = trackBarAccel1.Value;
                numericUpDownAccel2.Value = trackBarAccel2.Value;
                numericUpDownVelocity1.Value = trackBarVelocity1.Value;
                numericUpDownVelocity2.Value = trackBarVelocity2.Value;


                phidgetMotor.VelocityChange += new Phidgets.Events.VelocityChangeEventHandler(OnVelocityChange);
                phidgetMotor.InputChange += new Phidgets.Events.InputChangeEventHandler(OnInputChange);
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
                return false;
            }
            return true;
        }

        void OnVelocityChange(object sender, Phidgets.Events.VelocityChangeEventArgs e)
        {
            PrintDebug("{0}:{1:000.000}", e.Index, e.Velocity);
        }

        void OnInputChange(object sender, Phidgets.Events.InputChangeEventArgs e)
        {
            PrintDebug("{0}:{1}", e.Index, e.Value);

            if (e.Index >= phidgetMotor.inputs.Count)
            {
                return;
            }
            else
            {
                checkBoxInputs[e.Index].Checked = e.Value;
                Application.DoEvents();

                int newVal = 0;
                if (e.Value)
                    newVal = 1;

                Data.Motor motorData = new Data.Motor(SerialNumber,
                                                   Data.States.In,
                                                   Data.Motor.SubTypes.Digital,
                                                   e.Index,
                                                   newVal);
                CreateServerWriteEvent(motorData.ToArray());
            }

        }

        private void trackBarVelocity1_ValueChanged(object sender, EventArgs e)
        {
            numericUpDownVelocity1.Value = trackBarVelocity1.Value;
        }

        private void trackBarAccel1_ValueChanged(object sender, EventArgs e)
        {
            numericUpDownAccel1.Value = trackBarAccel1.Value;
            
        }

        private void trackBarVelocity2_ValueChanged(object sender, EventArgs e)
        {
            numericUpDownVelocity2.Value = trackBarVelocity2.Value;
        }

        private void trackBarAccel2_ValueChanged(object sender, EventArgs e)
        {
            numericUpDownAccel2.Value = trackBarAccel2.Value;
        }

        private void numericUpDownVelocity1_ValueChanged(object sender, EventArgs e)
        {
            if (phidgetMotor != null && phidgetMotor.Attached)
                phidgetMotor.motors[0].Velocity = (double)numericUpDownVelocity1.Value;
        }

        private void numericUpDownAccel1_ValueChanged(object sender, EventArgs e)
        {
            if (phidgetMotor != null && phidgetMotor.Attached)
                phidgetMotor.motors[0].Acceleration = (double)numericUpDownAccel1.Value;
        }

        private void numericUpDownVelocity2_ValueChanged(object sender, EventArgs e)
        {
            if (phidgetMotor != null && phidgetMotor.Attached)
                phidgetMotor.motors[1].Velocity = (double)numericUpDownVelocity2.Value;
        }

        private void numericUpDownAccel2_ValueChanged(object sender, EventArgs e)
        {
            if (phidgetMotor != null && phidgetMotor.Attached)
                phidgetMotor.motors[1].Acceleration = (double)numericUpDownAccel2.Value;
        }

        override public void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {
            try
            {
                if (CheckDeviceType(e.Args))
                {
                    Data.Motor m = Data.Motor.Parse(e.Args);

                    if (m != null)
                    {
                        if (m.Port < phidgetMotor.motors.Count &&
                            m.State == Data.States.Out)
                        {
                            switch (m.SubType)
                            {
                                case Data.Motor.SubTypes.Velocity:
                                    trackBarVelocities[m.Port].Value = m.Data;
                                    break;
                                case Data.Motor.SubTypes.Accel:
                                    trackBarAccels[m.Port].Value = m.Data;
                                    break;
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }



        #region Save/Load
       
        override public void SaveSetting()
        {
            StreamWriter fileStream = null;
            string file = GetSettingFile();

            try
            {
                fileStream = new StreamWriter(file, false);
                fileStream.WriteLine();

            }
            catch (IOException ex)
            {
                PrintError(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Saving " + Path.GetFileName(file));
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        override public void LoadSetting()
        {
            StreamReader fileStream = null;
            string file = GetSettingFile();

            try
            {
                if (File.Exists(file))
                {
                    fileStream = new StreamReader(file);

                    string line = null;
                    string[] args = null;

                    line = fileStream.ReadLine();
                    args = line.Split("\t".ToCharArray());
                }
            }
            catch (IOException ex)
            {
                PrintMessage(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Loading " + Path.GetFileName(file));

            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }


        #endregion



    }
}

