/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;

namespace MobiServer.Devices.Phidget.Control
{
    public partial class Servo : Base
    {
        TrackBar[] trackBarServos = null;
        Phidgets.Servo phidgetServo = null;

        public Servo(Phidgets.Phidget phidget)
            : base(phidget)
        {
            InitializeComponent();
            InitComponentArray();
            phidgetServo = (Phidgets.Servo)phidget;
            LoadSetting();
        }

       void InitComponentArray()
        {
            trackBarServos = new System.Windows.Forms.TrackBar[4];
            trackBarServos[0] = trackBarServo1;
            trackBarServos[1] = trackBarServo2;
            trackBarServos[2] = trackBarServo3;
            trackBarServos[3] = trackBarServo4;

            return;
        }

        override public bool Start()
        {
            try
            {
                if (!CheckDeviceAttached())
                {
                    return false;
                }

                PrintType();

                for (int i = 0; i < trackBarServos.Length; i++)
                {
                    if (i < phidgetServo.servos.Count)
                    {
                        trackBarServos[i].Enabled = true;
                        phidgetServo.servos[i].Position = trackBarServos[i].Value;
                    }
                    else
                    {
                        trackBarServos[i].Enabled = false;
                    }
                }
          
                phidgetServo.PositionChange += new Phidgets.Events.PositionChangeEventHandler(OnPositionChange);
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
                return false;
            }
            return true;
        }

        void OnPositionChange(object sender, Phidgets.Events.PositionChangeEventArgs args)
        {
            PrintDebug("{0}:{1:000.000}", args.Index, args.Position);

        }

        public override void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {
            try
            {
                if (CheckDeviceType(e.Args))
                {
                    Data.Servo s = Data.Servo.Parse(e.Args);

                    if (s != null)
                    {
                        if (s.Port < phidgetServo.servos.Count)
                        {
                            switch (s.State)
                            {
                                case Data.States.Out:
                                    trackBarServos[s.Port].Value = s.Data;
                                    break;
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }

        private void trackBarServo1_ValueChanged(object sender, System.EventArgs e)
        {
            numericUpDownServo1.Value = trackBarServo1.Value;
        }

        private void trackBarServo2_ValueChanged(object sender, System.EventArgs e)
        {
            numericUpDownServo2.Value = trackBarServo2.Value;
        }

        private void trackBarServo3_ValueChanged(object sender, System.EventArgs e)
        {
            numericUpDownServo3.Value = trackBarServo3.Value;
        }

        private void trackBarServo4_ValueChanged(object sender, System.EventArgs e)
        {
            numericUpDownServo4.Value = trackBarServo4.Value;
        }

        private void numericUpDownServo1_ValueChanged(object sender, EventArgs e)
        {
            if (phidgetServo != null && phidgetServo.Attached)
                phidgetServo.servos[0].Position = (double)numericUpDownServo1.Value;
        }

        private void numericUpDownServo2_ValueChanged(object sender, EventArgs e)
        {
            if (phidgetServo != null && phidgetServo.Attached && phidgetServo.servos.Count > 1)
                phidgetServo.servos[1].Position = (double)numericUpDownServo2.Value;
        }

        private void numericUpDownServo3_ValueChanged(object sender, EventArgs e)
        {
            if (phidgetServo != null && phidgetServo.Attached && phidgetServo.servos.Count > 1)
                phidgetServo.servos[2].Position = (double)numericUpDownServo3.Value;
        }

        private void numericUpDownServo4_ValueChanged(object sender, EventArgs e)
        {
            if (phidgetServo != null && phidgetServo.Attached && phidgetServo.servos.Count > 1)
                phidgetServo.servos[3].Position = (double)numericUpDownServo4.Value;
        }

        #region Save/Load

        override public void SaveSetting()
        {
            StreamWriter fileStream = null;
            string file = GetSettingFile();

            try
            {
                fileStream = new StreamWriter(file, false);

                for (int i = 0; i < trackBarServos.Length; i++)
                {
                    fileStream.Write("{0}\t", trackBarServos[i].Value);
                }
                fileStream.WriteLine();

            }
            catch (IOException ex)
            {
                PrintError(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Saving " + Path.GetFileName(file));
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        override public void LoadSetting()
        {
            StreamReader fileStream = null;
            string file = GetSettingFile();

            try
            {
                if (File.Exists(file))
                {

                    fileStream = new StreamReader(file);

                    string line = null;
                    string[] args = null;

                    line = fileStream.ReadLine();

                    args = line.Split("\t".ToCharArray());

                    for (int i = 0; i < trackBarServos.Length; i++)
                    {
                        trackBarServos[i].Value = int.Parse(args[i]);
                    }

                }

            }
            catch (IOException ex)
            {
                PrintMessage(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Loading " + Path.GetFileName(file));

            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }
        #endregion

    }
}

