/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;

namespace MobiServer.Devices.Phidget.Control
{
    public partial class Temperature : Base
    {
        Phidgets.TemperatureSensor phidgetTemperature = null;
        public Temperature(Phidgets.Phidget phidget)
            : base(phidget)
        {
            InitializeComponent();
            phidgetTemperature = (Phidgets.TemperatureSensor)phidget;
            LoadSetting();
        }

        override public bool Start()
        {
            try
            {
                if (!CheckDeviceAttached())
                    return false;
               
                PrintType();
                phidgetTemperature.thermocouples[0].Sensitivity = (double)numericUpDownThreshold.Value;
                phidgetTemperature.TemperatureChange +=new Phidgets.Events.TemperatureChangeEventHandler(OnTemperatureChange);               
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
                return false;
            }
            return true;
        }

        void OnTemperatureChange(object sender, Phidgets.Events.TemperatureChangeEventArgs args)
        {
            
            double data = Math.Round(args.Temperature - (double)numericUpDownZero.Value, 2);
            double data2 = Math.Round(phidgetTemperature.ambientSensor.Temperature - (double)numericUpDownZero.Value, 2);
            PrintDebug("{0:000.00} || {1:000.00}", data, data2);

            numericUpDownValueThermocouple.Value = (decimal)data;
            numericUpDownValueAmbient.Value = (decimal)data2;

            Data.Temperature temperature = new Data.Temperature(SerialNumber, Data.States.In,data, data2);
            CreateServerWriteEvent(temperature.ToArray());                                                                    

        }

        private void numericUpDownThreshold_ValueChanged(object sender, EventArgs e)
        {
            if (phidgetTemperature != null && phidgetTemperature.Attached)
            {
                phidgetTemperature.thermocouples[0].Sensitivity = (double)numericUpDownThreshold.Value;
            }
        }
      
        public override void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {
            try
            {
                if (CheckDeviceType(e.Args))
                {
                    Data.Temperature t = Data.Temperature.Parse(e.Args);
                    switch (t.State)
                    {
                        case Data.States.Set:

                            switch (t.SubType)
                            {
                                case Data.Temperature.SubTypes.Threshold:
                                    numericUpDownThreshold.Value = (decimal)t.Data;
                                    break;
                                case Data.Temperature.SubTypes.Zero:
                                    numericUpDownZero.Value = (decimal)t.Data;
                                    break;
                            }
                            break;
                    }
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }


        #region Save/Load
        override public void SaveSetting()
        {
            StreamWriter fileStream = null;
            string file = GetSettingFile();

            try
            {
                fileStream = new StreamWriter(file, false);
                fileStream.WriteLine("{0}", numericUpDownThreshold.Value);
                fileStream.WriteLine();

            }
            catch (IOException ex)
            {
                PrintError(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Saving " + Path.GetFileName(file));
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        override public void LoadSetting()
        {
            StreamReader fileStream = null;
            string file = GetSettingFile();

            try
            {
                if (File.Exists(file))
                {

                    fileStream = new StreamReader(file);

                    string line = null;

                    line = fileStream.ReadLine();

                    numericUpDownThreshold.Value = decimal.Parse(line);
                }

            }
            catch (IOException ex)
            {
                PrintMessage(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Loading " + Path.GetFileName(file));

            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }


        #endregion


    }
}

