/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using System.Text.RegularExpressions;

namespace MobiServer.Devices.Phidget.Control
{
    public partial class TextLCD : Base
    {
 
        TextBox[] textBoxRaws = null;
        Phidgets.TextLCD phidgetTextLCD = null;

        public TextLCD(Phidgets.Phidget phidget)
            : base(phidget)
        {
            InitializeComponent();
            InitComponentArray();
            phidgetTextLCD = (Phidgets.TextLCD)phidget;
            LoadSetting();     
        }

        void InitComponentArray()
        {
            textBoxRaws = new TextBox[2];
            textBoxRaws[0] = textBoxRaw1;
            textBoxRaws[1] = textBoxRaw2;
        }
     
        override public bool Start()
        {
            try
            {
                if (!CheckDeviceAttached())
                    return false;

                PrintType();

                phidgetTextLCD.Backlight = checkBoxBacklight.Checked;
                phidgetTextLCD.Cursor = checkBoxCursorStill.Checked;
                phidgetTextLCD.CursorBlink = checkBoxCursorBlink.Checked;

                phidgetTextLCD.Contrast = trackBarContrast.Value;

                phidgetTextLCD.rows[0].DisplayString = textBoxRaw1.Text;
                phidgetTextLCD.rows[1].DisplayString = textBoxRaw2.Text;

            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
                return false;
            }
            return true;
        }



        private void textBoxRaw1_TextChanged(object sender, EventArgs e)
        {
            if (phidgetTextLCD != null && phidgetTextLCD.Attached)
            {
                phidgetTextLCD.rows[0].DisplayString = textBoxRaw1.Text;
            }
        }

        private void textBoxRaw2_TextChanged(object sender, EventArgs e)
        {
            if (phidgetTextLCD != null && phidgetTextLCD.Attached)
            {
                phidgetTextLCD.rows[1].DisplayString = textBoxRaw2.Text;
            }
        }

        private void trackBarContrast_ValueChanged(object sender, EventArgs e)
        {
            
            if (numericUpDownContrast.Maximum >= (decimal)trackBarContrast.Value)
            {
                numericUpDownContrast.Value = (decimal)trackBarContrast.Value;
            }
            else
            {
                numericUpDownContrast.Value = numericUpDownContrast.Maximum;
            }

            if (phidgetTextLCD != null && phidgetTextLCD.Attached)
            {
                phidgetTextLCD.Contrast = (int)numericUpDownContrast.Value;
            }
        }

        private void checkBoxBacklight_CheckedChanged(object sender, EventArgs e)
        {
            if (phidgetTextLCD != null && phidgetTextLCD.Attached)
            {
                phidgetTextLCD.Backlight = checkBoxBacklight.Checked;
            }
        }

        override public void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {
            try
            {
                if (CheckDeviceType(e.Args))
                {

                    Data.TextLCD lcd = Data.TextLCD.Parse(e.Args);

                    if (lcd != null)
                    {
                         switch (lcd.State)
                        {
                            case Data.States.Set:
                                switch(lcd.SubType)
                                {
                                    case Data.TextLCD.SubTypes.Contrast:
                                        trackBarContrast.Value = lcd.Data;
                                        break;
                                    case Data.TextLCD.SubTypes.Backlight:
                                        if (lcd.Data == 1)
                                        {
                                            checkBoxBacklight.Checked = true;
                                        }
                                        else if (lcd.Data == 0)
                                        {
                                            checkBoxBacklight.Checked = false;
                                        }
                                        break;
                                    case Data.TextLCD.SubTypes.CursorStill:
                                        if (lcd.Data == 1)
                                        {
                                            checkBoxCursorStill.Checked = true;
                                        }
                                        else if (lcd.Data == 0)
                                        {
                                            checkBoxCursorStill.Checked = false;
                                        }
                                        break;
                                    case Data.TextLCD.SubTypes.CursorBlink:
                                        if (lcd.Data == 1)
                                        {
                                            checkBoxCursorBlink.Checked = true;
                                        }
                                        else if (lcd.Data == 0)
                                        {
                                            checkBoxCursorBlink.Checked = false;
                                        }
                                        break;
                                    case Data.TextLCD.SubTypes.Cursor:
                                        if (lcd.Data == 1)
                                        {
                                            checkBoxCursorBlink.Checked = true;
                                            checkBoxCursorStill.Checked = true;
                                        }
                                        else if (lcd.Data == 0)
                                        {
                                            checkBoxCursorBlink.Checked = false;
                                            checkBoxCursorStill.Checked = false;
                                        }
                                        break;                              
                                }     
                                break;
                            case Data.States.Out:
                                if(lcd.Port < textBoxRaws.Length){
                                    textBoxRaws[lcd.Port].Text = lcd.Line;
                                }    
                                break;
                         }
                    }
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }

        private void checkBoxCursorStill_CheckedChanged(object sender, EventArgs e)
        {
            if (phidgetTextLCD != null && phidgetTextLCD.Attached)
            {
                phidgetTextLCD.Cursor = checkBoxCursorStill.Checked;
            }
        }

        private void checkBoxCursorBlink_CheckedChanged(object sender, EventArgs e)
        {
            if (phidgetTextLCD != null && phidgetTextLCD.Attached)
            {
                phidgetTextLCD.CursorBlink = checkBoxCursorBlink.Checked;
            }
        }

        #region Save/Load

        override public void SaveSetting()
        {
            StreamWriter fileStream = null;
            string file = GetSettingFile();

            try
            {
                fileStream = new StreamWriter(file, false);

                fileStream.WriteLine(textBoxRaw1.Text);
                fileStream.WriteLine(textBoxRaw2.Text);
                fileStream.WriteLine(checkBoxBacklight.Checked);
                fileStream.WriteLine(checkBoxCursorStill.Checked);
                fileStream.WriteLine(checkBoxCursorBlink.Checked);
                fileStream.WriteLine(trackBarContrast.Value);

            }
            catch (IOException ex)
            {
                PrintError(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Saving " + Path.GetFileName(file));
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        override public void LoadSetting()
        {
            StreamReader fileStream = null;
            string file = GetSettingFile();

            try
            {
                if (File.Exists(file))
                {

                    fileStream = new StreamReader(file);

                    string line = null;
                    line = fileStream.ReadLine();
                    textBoxRaw1.Text = line;
                    line = fileStream.ReadLine();
                    textBoxRaw2.Text = line;
                    line = fileStream.ReadLine();
                    checkBoxBacklight.Checked = bool.Parse(line);
                    line = fileStream.ReadLine();
                    checkBoxCursorStill.Checked = bool.Parse(line);
                    line = fileStream.ReadLine();
                    checkBoxCursorBlink.Checked = bool.Parse(line);
                    line = fileStream.ReadLine();
                    trackBarContrast.Value = int.Parse(line);
                    numericUpDownContrast.Value = (int)trackBarContrast.Value;
                }

            }
            catch (IOException ex)
            {
                PrintMessage(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Loading " + Path.GetFileName(file));

            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        #endregion

    }
}

