/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;

namespace MobiServer.Devices.Phidget.Control
{
    public partial class Weight : Base
    {

        Phidgets.WeightSensor phidgetWeight = null;

        public Weight(Phidgets.Phidget phidget)
            : base(phidget)
        {
            InitializeComponent();
            phidgetWeight = (Phidgets.WeightSensor)phidget;
            LoadSetting();
        }

        override public bool Start()
        {
            try
            {
                if (!CheckDeviceAttached())
                {
                    return false;
                }

                PrintType();
               
                phidgetWeight.sensor.Sensitivity = (double)numericUpDownWeightThreshold.Value;
                phidgetWeight.WeightChange += new Phidgets.Events.WeightChangeEventHandler(OnWeightChange);
            }

            catch (Exception ex)
            {
                PrintError(ex.Message);
                return false;
            }
            return true;
        }

        void OnWeightChange(object sender, Phidgets.Events.WeightChangeEventArgs args)
        {
            double w = Math.Round(args.Weight - (double)numericUpDownZero.Value,2);
            PrintDebug("Weight:{0}", w);
            numericUpDownWeight.Value = (decimal)w;

            Data.Weight weight = new Data.Weight(SerialNumber, Data.States.In, w);
            CreateServerWriteEvent(weight.ToArray());
            Application.DoEvents();

        }

        private void numericUpDownWeightThreshold_ValueChanged(object sender, System.EventArgs e)
        {
            if (phidgetWeight != null && phidgetWeight.Attached)
            {
                phidgetWeight.sensor.Sensitivity = (double)numericUpDownWeightThreshold.Value;
            }
        }

        public override void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {
            try
            {
                if (CheckDeviceType(e.Args))
                {
                    Data.Weight w = Data.Weight.Parse(e.Args);
                    switch (w.State)
                    {
                        case Data.States.Set:
                            switch (w.SubType)
                            {
                                case Data.Weight.SubTypes.Threshold:
                                    numericUpDownWeightThreshold.Value = (decimal)w.Data;
                                    break;
                                case Data.Weight.SubTypes.Zero:
                                    numericUpDownZero.Value = (decimal)w.Data;
                                    break;
                            }
                            break;
                    }
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }

        private void numericUpDownWeight_ValueChanged(object sender, EventArgs e)
        {

        }

        private void buttonAjust_Click(object sender, EventArgs e)
        {
            numericUpDownZero.Value = (decimal)(numericUpDownZero.Value +numericUpDownWeight.Value);    //phidgetWeight.sensor.Weight
            numericUpDownWeight.Value = (decimal)0.0;
        }

        private void buttonClear_Click(object sender, EventArgs e)
        {         
            numericUpDownWeight.Value = (decimal)(numericUpDownWeight.Value + numericUpDownZero.Value);
            numericUpDownZero.Value = (decimal)0.0;
        }

        private void numericUpDownZero_ValueChanged(object sender, EventArgs e)
        {
        }

        #region Save/Load

        override public void SaveSetting()
        {
            StreamWriter fileStream = null;
            string file = GetSettingFile();

            try
            {
                fileStream = new StreamWriter(file, false);
                fileStream.WriteLine("{0}", numericUpDownWeightThreshold.Value);
                fileStream.WriteLine("{0}", numericUpDownZero.Value);

            }
            catch (IOException ex)
            {
                PrintError(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Saving " + Path.GetFileName(file));
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        override public void LoadSetting()
        {
            StreamReader fileStream = null;
            string file = GetSettingFile();
            try
            {
                if (File.Exists(file))
                {

                    fileStream = new StreamReader(file);

                    string line = null;

                    line = fileStream.ReadLine();
                    numericUpDownWeightThreshold.Value = decimal.Parse(line);

                    line = fileStream.ReadLine();
                    numericUpDownZero.Value = decimal.Parse(line);
                }

            }
            catch (IOException ex)
            {
                PrintMessage(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Loading " + Path.GetFileName(file));

            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }


        #endregion
    }
}

