/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Text;

using System.Text.RegularExpressions;

namespace MobiServer.Devices.Phidget.Data
{
    public class Accel : Base
    {
        int port = 0;
        double data = 0.0;
        string subType = "";

        public class SubTypes
        {
            public const string Threshold = "Threshold";
            public const string Zero = "Zero";
        }

        public Accel(int serialNumber, string state, int port, double data)
            : base(Types.Accel, serialNumber, state)
        {
            this.port = port;
            this.data = data;
        }

        public Accel(int serialNumber, string state, string subType, int port, double data)
            : base(Types.Accel, serialNumber, state)
        {
            this.subType = GetSubType(subType);
            this.port = port;
            this.data = data;
        }

        public Accel(int serialNumber, string state)
            : base(Types.Accel, serialNumber, state)
        {


        }

        string GetSubType(string type)
        {
            Regex regex = new Regex(type, RegexOptions.IgnoreCase);

            if (regex.Match(SubTypes.Threshold).Success)
                type = SubTypes.Threshold;
            else if (regex.Match(SubTypes.Zero).Success)
                type = SubTypes.Zero;

            return type;
        }

        public static Accel Parse(string []args)
        {
            try
            {
                //string[] args = line.Split(DELIMITER_ARGS.ToCharArray());
                int serialNumber = ParseSerialNumber(args);
                string state = ParseState(args);
                string subType = args[4];
                int port = int.Parse(args[5]);
                double data = double.Parse(args[6]);

                Accel accelData = new Accel(serialNumber, state, subType, port, data);
                return accelData;
            }
            catch (Exception ex)
            {
                return null;
            }
        }

        public string[] ToArray()
        {
            string[] args = null;
            string[] headers = GetHeader();
            switch (state)
            {
                case States.Attach:
                case States.Detach:
                    args = base.ToArray();
                    break;
                case States.In:
                    int length = headers.Length;
                    args = new string[length + 2];
                    headers.CopyTo(args, 0);
                    args[length + 0] = port.ToString();
                    args[length + 1] = String.Format("{0:00.00}",data);
                    break;
            }
            return args;
        }

        public int Port
        {
            get { return port; }
        }

        public double Data
        {
            get { return data; }
        }

        public string SubType
        {
            get { return subType; }

        }

    }
}
