/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Text;

namespace MobiServer.Devices.Phidget.Data
{
    public class Servo : Base
    {
        int port = 0;
        int data = 0;

        public const int PORT_MAX = 4;

        public Servo(int serialNumber, string state, int port, int data)
            :
                base(Types.Servo, serialNumber, state)
        {

            this.port = port;
            this.data = data;
        }

        public Servo(int serialNumber, string state)
            :
            base(Types.Servo, serialNumber, state)
        {
        }

        public override string ToString()
        {
            string str = "";
            switch (state)
            {
                case States.Attach:
                case States.Detach:
                    str = base.ToString();
                    break;
            }
            return str;
        }

        public int Port
        {
            get { return port; }
        }

        public int Data
        {
            get { return data; }
        }

        public static Servo Parse(string []args)
        {
            try
            {
                //string[] args = line.Split(DELIMITER_ARGS.ToCharArray());
                int serialNumber = ParseSerialNumber(args);
                string state = ParseState(args);

                int port = int.Parse(args[4]);
                int data = int.Parse(args[5]);
                Servo servo =
                    new Servo(serialNumber, state, port, data);
                return servo;
            }
            catch
            {
                return null;
            }

        }

    }
}
