/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace MobiServer.Devices.Phidget.Data
{
    public class TextLCD : Base
    {

        public class SubTypes
        {
            public const string Contrast = "Contrast";
            public const string Backlight = "Backlight";
            public const string Cursor = "Cursor";
            public const string CursorStill = "CursorStill";
            public const string CursorBlink = "CursorBlink";
        }

        int port = 0;
        string line = "";

        string subType = "";
        int data = 0;

        public TextLCD(int serialNumber, string state, int port, string line)
            : base(Types.TextLCD, serialNumber, state)
        {
            this.port = port;
            this.line = line;
        }

        public TextLCD(int serialNumber, string state, string subType, int data)
            : base(Types.TextLCD, serialNumber, state)
        {
            this.subType = GetSubType(subType);
            this.data = data;
        }

        string GetSubType(string type)
        {
            Regex regex = new Regex(type, RegexOptions.IgnoreCase);

            if (regex.Match(SubTypes.Backlight).Success)
                type = SubTypes.Backlight;
            else if (regex.Match(SubTypes.Contrast).Success)
                type = SubTypes.Contrast;
            else if (regex.Match(SubTypes.Cursor).Success)
                type = SubTypes.Cursor;
            else if (regex.Match(SubTypes.CursorBlink).Success)
                type = SubTypes.CursorBlink;
            else if (regex.Match(SubTypes.CursorStill).Success)
                type = SubTypes.CursorStill;

            return type;
        }

        public TextLCD(int serialNumber, string state)
            :
            base(Types.TextLCD, serialNumber, state)
        {
        }

        public override string ToString()
        {
            string str = "";
            switch (state)
            {
                case States.Attach:
                case States.Detach:
                    str = base.ToString();
                    break;
            }
            return str;
        }

        public int Port
        {
            get { return port; }
        }

        public int Data
        {
            get { return data; }
        }

        public string Line
        {

            get { return line; }
        }

        public string SubType
        {
            get { return subType; }

        }

        public static TextLCD Parse(string [] args)
        {
            try
            {
                TextLCD textLCD = null;
                //string[] args = line.Split(DELIMITER_ARGS.ToCharArray());

                int serialNumber = ParseSerialNumber(args);
                string state = ParseState(args);

                switch (state)
                {
                    case States.Out:
                        int port = int.Parse(args[4]);
                        string l = "";
                        for (int i = 5; i < args.Length; i++)
                        {
                            l += args[i] + " ";
                        }

                        l = Regex.Replace(l, @"\t", " ");
                        l = l.Remove(l.Length - 1);

                        textLCD = new TextLCD(serialNumber, state, port, l);
                        break;
                    case States.Set:
                        string subType = args[4];
                        int data = int.Parse(args[5]);
                        textLCD = new TextLCD(serialNumber, state, subType, data);
                        break;

                }

                return textLCD;

            }
            catch
            {
                return null;
            }

        }

    }
}
