/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Drawing;
using System.Collections;

using Phidgets;

namespace MobiServer
{
    public class PhidgetManager
    {
        Hashtable phidgetHolder;
        MobiServer.PhidgetForm form = null;

        public PhidgetManager(MobiServer.PhidgetForm form)
        {
            this.form = form;

            phidgetHolder = new Hashtable();
            
                InitInterfaceKit();
                InitAccel();
                InitRFID();
                InitServo();
                InitWeight();

                InitTemperature();
                InitTextLCD();
                InitMotor();
                InitEncoder();
                InitPH();
            
        }

        void OnAttach(object sender, Phidgets.Events.AttachEventArgs e)
        {
            LoadControl(sender, e.Device, true);
            form.PrintMessage("Attached: {0}:{1}", e.Device.Type, e.Device.SerialNumber);
        }

        void OnDetach(object sender, Phidgets.Events.DetachEventArgs e)
        {
            LoadControl(sender, e.Device, false);
            form.PrintMessage("Detached: {0}:{1}", e.Device.Type, e.Device.SerialNumber);
        }

        void LoadControl(object sender, Phidgets.Phidget phidget, bool attached)
        {
            string attach = Devices.Phidget.Data.States.Attach;
            string detach = Devices.Phidget.Data.States.Detach;

            switch (phidget.Type)
            {
                case MobiServer.Devices.Phidget.Data.Types.InterfaceKit:
                    Phidgets.InterfaceKit interfaceKit = (Phidgets.InterfaceKit)sender;
                    Devices.Phidget.Control.InterfaceKit interfaceControl = new Devices.Phidget.Control.InterfaceKit(interfaceKit);//server

                    if (attached)
                    {
                        form.StartControl(interfaceControl);
                        InitInterfaceKit();
                        form.WriteAllSockets(new Devices.Phidget.Data.InterfaceKit(phidget.SerialNumber, attach).ToArray());
                    }
                    else
                    {
                        form.DisableControl(interfaceControl);
                        form.WriteAllSockets(new Devices.Phidget.Data.InterfaceKit(phidget.SerialNumber, detach).ToArray());
                    }
                    break;
                case MobiServer.Devices.Phidget.Data.Types.Accel:
                    Phidgets.Accelerometer accel = (Phidgets.Accelerometer)sender;

                    Devices.Phidget.Control.Accel accelControl = new Devices.Phidget.Control.Accel(accel);

                    if (attached)
                    {
                        form.StartControl(accelControl);
                        InitAccel();
                        form.WriteAllSockets(new Devices.Phidget.Data.Accel(phidget.SerialNumber, attach).ToArray());
                    }
                    else
                    {
                        form.DisableControl(accelControl);
                        form.WriteAllSockets(new Devices.Phidget.Data.Accel(phidget.SerialNumber, detach).ToArray());
                    }
                    break;
                case MobiServer.Devices.Phidget.Data.Types.RFID:
                    Phidgets.RFID rfid = (Phidgets.RFID)sender;

                    Devices.Phidget.Control.RFID rfidControl = new Devices.Phidget.Control.RFID(rfid);

                    if (attached)
                    {
                        form.StartControl(rfidControl);
                        InitRFID();
                        form.WriteAllSockets(new Devices.Phidget.Data.RFID(phidget.SerialNumber, attach).ToArray());
                    }
                    else
                    {
                        form.DisableControl(rfidControl);
                        form.WriteAllSockets(new Devices.Phidget.Data.RFID(phidget.SerialNumber, detach).ToArray());
                    }

                    break;

                case MobiServer.Devices.Phidget.Data.Types.Servo:
                    Phidgets.Servo servo = (Phidgets.Servo)sender;

                    Devices.Phidget.Control.Servo servoControl = new Devices.Phidget.Control.Servo(servo);

                    if (attached)
                    {
                        form.StartControl(servoControl);
                        InitServo();
                        form.WriteAllSockets(new Devices.Phidget.Data.Servo(phidget.SerialNumber, attach).ToArray());
                    }
                    else
                    {
                        form.DisableControl(servoControl);
                        form.WriteAllSockets(new Devices.Phidget.Data.Servo(phidget.SerialNumber, detach).ToArray());
                    }
                    break;

                case MobiServer.Devices.Phidget.Data.Types.Weight:
                    Phidgets.WeightSensor weight = (Phidgets.WeightSensor)sender;

                    Devices.Phidget.Control.Weight weightControl = new Devices.Phidget.Control.Weight(weight);

                    if (attached)
                    {
                        form.StartControl(weightControl);
                        InitWeight();
                        form.WriteAllSockets(new Devices.Phidget.Data.Weight(phidget.SerialNumber, attach).ToArray());
                    }
                    else
                    {
                        form.DisableControl(weightControl);
                        form.WriteAllSockets(new Devices.Phidget.Data.Weight(phidget.SerialNumber, detach).ToArray());
                    }
                    break;

                case MobiServer.Devices.Phidget.Data.Types.Temperature:
                    Phidgets.TemperatureSensor temperature = (Phidgets.TemperatureSensor)sender;
                    Devices.Phidget.Control.Temperature temperatureControl = new Devices.Phidget.Control.Temperature(temperature);

                    if (attached)
                    {
                        form.StartControl(temperatureControl);
                        InitTemperature();
                        form.WriteAllSockets(new Devices.Phidget.Data.Temperature(phidget.SerialNumber, attach).ToArray());
                    }
                    else
                    {
                        form.DisableControl(temperatureControl);
                        form.WriteAllSockets(new Devices.Phidget.Data.Temperature(phidget.SerialNumber, detach).ToArray());
                    }
                    break;
                case MobiServer.Devices.Phidget.Data.Types.Motor:
                    Phidgets.MotorControl motor = (Phidgets.MotorControl)sender;

                    Devices.Phidget.Control.Motor motorControl = new Devices.Phidget.Control.Motor(motor);

                    if (attached)
                    {
                        form.StartControl(motorControl);
                        InitMotor();
                        form.WriteAllSockets(new Devices.Phidget.Data.Motor(phidget.SerialNumber, attach).ToArray());
                    }
                    else
                    {
                        form.DisableControl(motorControl);
                        form.WriteAllSockets(new Devices.Phidget.Data.Motor(phidget.SerialNumber, detach).ToArray());
                    }
                    break;
                case MobiServer.Devices.Phidget.Data.Types.PH:
                    Phidgets.PHSensor phSensor = (Phidgets.PHSensor)sender;

                    Devices.Phidget.Control.PH phControl = new Devices.Phidget.Control.PH(phSensor);

                    if (attached)
                    {
                        form.StartControl(phControl);
                        InitPH();
                    }
                    else
                    {
                        form.DisableControl(phControl);
                    }
                    break;
                case MobiServer.Devices.Phidget.Data.Types.TextLCD:
                    Phidgets.TextLCD textLCD = (Phidgets.TextLCD)sender;

                    Devices.Phidget.Control.TextLCD textLCDControl = new Devices.Phidget.Control.TextLCD(textLCD);

                    if (attached)
                    {
                        form.StartControl(textLCDControl);
                        InitTextLCD();
                        form.WriteAllSockets(new Devices.Phidget.Data.TextLCD(phidget.SerialNumber, attach).ToArray());
                    }
                    else
                    {
                        form.DisableControl(textLCDControl);
                        form.WriteAllSockets(new Devices.Phidget.Data.TextLCD(phidget.SerialNumber, detach).ToArray());
                    }
                    break;

                case MobiServer.Devices.Phidget.Data.Types.Encoder:
                    Phidgets.Encoder encoder = (Phidgets.Encoder)sender;

                    Phidgets.Phidget p =new Phidgets.Phidget();
                    
                    Devices.Phidget.Control.Encoder encoderControl = new Devices.Phidget.Control.Encoder(encoder);

                    if (attached)
                    {
                        form.StartControl(encoderControl);
                        InitEncoder();
                    }
                    else
                    {
                        form.DisableControl(encoderControl);
                    }
                    break;
            }

        }
        void InitInterfaceKit()
        {
            
            Phidgets.InterfaceKit phidgetInterfaceKit = null;

            phidgetInterfaceKit = new Phidgets.InterfaceKit();
            phidgetInterfaceKit.Attach += new Phidgets.Events.AttachEventHandler(OnAttach);
            phidgetInterfaceKit.Detach += new Phidgets.Events.DetachEventHandler(OnDetach);
            phidgetInterfaceKit.open();
            phidgetHolder.Add(phidgetHolder.Count, phidgetInterfaceKit);
        }

        void InitRFID()
        {
            Phidgets.RFID phidgetRFID = null;
            phidgetRFID = new Phidgets.RFID();
            phidgetRFID.Attach += new Phidgets.Events.AttachEventHandler(OnAttach);
            phidgetRFID.Detach += new Phidgets.Events.DetachEventHandler(OnDetach);
            phidgetRFID.open();
            phidgetHolder.Add(phidgetHolder.Count, phidgetRFID);
        }

        void InitAccel()
        {
            Phidgets.Accelerometer phidgetAccel = null;
            phidgetAccel = new Phidgets.Accelerometer();
            phidgetAccel.Attach += new Phidgets.Events.AttachEventHandler(OnAttach);
            phidgetAccel.Detach += new Phidgets.Events.DetachEventHandler(OnDetach);
            phidgetAccel.open();
            phidgetHolder.Add(phidgetHolder.Count, phidgetAccel);
        }

        void InitServo()
        {
            Phidgets.Servo phidgetServo = null;
            phidgetServo = new Phidgets.Servo();
            phidgetServo.Attach += new Phidgets.Events.AttachEventHandler(OnAttach);
            phidgetServo.Detach += new Phidgets.Events.DetachEventHandler(OnDetach);
            phidgetServo.open();
            phidgetHolder.Add(phidgetHolder.Count, phidgetServo);
        }


        void InitWeight()
        {
            Phidgets.WeightSensor phidgetWeight = null;
            phidgetWeight = new Phidgets.WeightSensor();
            phidgetWeight.Attach += new Phidgets.Events.AttachEventHandler(OnAttach);
            phidgetWeight.Detach += new Phidgets.Events.DetachEventHandler(OnDetach);
            phidgetWeight.open();
            phidgetHolder.Add(phidgetHolder.Count, phidgetWeight);
        }

        void InitEncoder()
        {
            Phidgets.Encoder phidgetEncoder = null;
            phidgetEncoder = new Phidgets.Encoder();
            phidgetEncoder.Attach += new Phidgets.Events.AttachEventHandler(OnAttach);
            phidgetEncoder.Detach += new Phidgets.Events.DetachEventHandler(OnDetach);
            phidgetEncoder.open();
            phidgetHolder.Add(phidgetHolder.Count, phidgetEncoder);
        }

        void InitTextLCD()
        {
            Phidgets.TextLCD phidgetTextLcd = null;
            phidgetTextLcd = new Phidgets.TextLCD();
            phidgetTextLcd.Attach += new Phidgets.Events.AttachEventHandler(OnAttach);
            phidgetTextLcd.Detach += new Phidgets.Events.DetachEventHandler(OnDetach);
            phidgetTextLcd.open();
            phidgetHolder.Add(phidgetHolder.Count, phidgetTextLcd);
        }

        void InitTemperature()
        {
            Phidgets.TemperatureSensor phidgetTemperature = null;
            phidgetTemperature = new Phidgets.TemperatureSensor();
            phidgetTemperature.Attach += new Phidgets.Events.AttachEventHandler(OnAttach);
            phidgetTemperature.Detach += new Phidgets.Events.DetachEventHandler(OnDetach);
            phidgetTemperature.open();
            phidgetHolder.Add(phidgetHolder.Count, phidgetTemperature);
        }

        void InitMotor()
        {
            Phidgets.MotorControl phidgetMotorControl = null;
            phidgetMotorControl = new Phidgets.MotorControl();
            phidgetMotorControl.Attach += new Phidgets.Events.AttachEventHandler(OnAttach);
            phidgetMotorControl.Detach += new Phidgets.Events.DetachEventHandler(OnDetach);
            phidgetMotorControl.open();
            phidgetHolder.Add(phidgetHolder.Count, phidgetMotorControl);
        }

        void InitPH()
        {
            Phidgets.PHSensor phidgetPHSensor = null;
            phidgetPHSensor = new Phidgets.PHSensor();
            phidgetPHSensor.Attach += new Phidgets.Events.AttachEventHandler(OnAttach);
            phidgetPHSensor.Detach += new Phidgets.Events.DetachEventHandler(OnDetach);
            phidgetPHSensor.open();
            phidgetHolder.Add(phidgetHolder.Count, phidgetPHSensor);
        }

    }
}
